unit DirIters;

interface

uses
  Classes, FileCtrl;

type
  TFileEvent = procedure (Sender: TObject; const FileName: string; Attributes: TFileType) of object;

  TDirectoryIterator = class(TComponent)
  private
    FDirectory: string;
    FSearchMask: string;
    FSubdirectories: Boolean;
    FFileType: TFileType;
    FOnFile: TFileEvent;
    FOnFailure: TNotifyEvent;
  protected
    function SearchDirectory(const Dir, Mask: string): Boolean;
  public
    function Execute: Boolean;
  published
    property Directory: string read FDirectory write FDirectory;
    property FileType: TFileType read FFileType write FFileType;
    property SearchMask: string read FSearchMask write FSearchMask;
    property SubDirectories: Boolean read FSubDirectories write FSubDirectories;
    property OnFile: TFileEvent read FOnFile write FOnFile;
    property OnFailure: TNotifyEvent read FOnFailure write FOnFailure;
  end;

implementation

uses
  Windows, SysUtils, Dialogs;

{ ... TDirectoryIterator ... }

function ByteToFileType(Attr: Byte): TFileType;
begin
  Result := TFileType(Attr);
end;

function TDirectoryIterator.SearchDirectory(const Dir, Mask: string): Boolean;
var
  SR: TSearchRec;
  Found: Integer;
  SearchResult: Boolean;
  Attr: Byte;
begin
  if ftNormal in FFileType then
    Attr := faAnyFile
  else
    Attr := Byte(FFileType);
  Found := FindFirst(Dir + '\' + Mask, Attr, SR);
  Result := Found = 0;
  while Found = 0 do
  begin
    if Assigned(FOnFile) then
      FOnFile(Self, Dir + '\' + SR.Name, ByteToFileType(SR.Attr));
    Found := FindNext(SR);
  end;
  FindClose(SR);

  if not SubDirectories then Exit;

  Found := FindFirst(Dir + '\*.*', faDirectory, SR);
  while Found = 0 do
  begin
    if (SR.Attr and faDirectory <> 0) and
       (SR.Name <> '.') and
       (SR.Name <> '..') then
    begin
      SearchResult := SearchDirectory(Dir + '\' + SR.Name, Mask);
      Result := Result or SearchResult;
    end;
    Found := FindNext(SR);
  end;
  FindClose(SR);
end;

function TDirectoryIterator.Execute: Boolean;
begin
  Result := SearchDirectory(FDirectory, FSearchMask);
  if not Result and Assigned(FOnFailure) then
    FOnFailure(Self);
end;

end.
